;*_____________________________________________________________________________
;*
;*           Author: Terry Carmen 
;* Released to the Public Domain 4/17/1996
;*_____________________________________________________________________________
;*
;*
;*     Program Name:   isdir.asm 
;*        $Revision:   1.2  $
;*            $Date:   17 Apr 1996  9:44:36  $
;*
;*
;*
;* Revision History:
;*             $Log:   D:/prog/isdir/isdir.asv  $
;*
;*   Rev 1.2   17 Apr 1996  9:44:36
;*removed int 3 used for debugging
;*  
;*     Rev 1.1   17 Apr 1996  9:30:54
;*  moved description to header
;*  
;*    Rev 1.0   17 Apr 1996  9:28:12
;* Initial revision.
;*
;*_____________________________________________________________________________


;* Description:

;* assemble with TASM isdir /mx /w2

;* Clipper-callable function to determine if a given filespec is a directory
;* or not.
;*  
;* Returns .T. if the filespec is a directory, and .F. if it's anything else
;* (doesn't exist, is a file, volume label, etc.)
;* 
;* Will find directories marked hidden and/or system
;* 
;* Usage:
;* 
;* lRet :=IsDir(cDirectory)
;* 
;* where cDirectory is the name of the directory to search for, including
;* an optional path.


public ISDIR

extrn   __parinfo:far
extrn   __parclen:far
extrn   __parc:far
extrn   __retl:far

; some constants for DOS

READONLY     equ   1h   
HIDDEN       equ   2h   
SYSTEM       equ   4h   
VOLUMELABEL  equ   8h   
DIRECTORY    equ  10h   
ARCHIVE      equ  20h   


assume cs:_prog, ds:datasg, es:nothing

dgroup  group   datasg
datasg  segment public  para 'DATA'

  ; if we had data, this would be a good place for it 8-)

datasg ends



_prog segment 'CODE'

ISDIR  Proc  far
                push    bp
                mov     bp,sp
                push    ds
                push    es
                push    si
                push    di

                mov     ax, 0
                push    ax
                call    __parinfo           ; how many params?
                add     sp,2
                cmp     al,1                ; should be 1
                jne     @@bad_exit          ; if not, we're out of here..

                mov     ax, 1               ; check the type
                push    ax
                call    __parinfo
                add     sp,2
                and     ax,1                ; is it a string?
                cmp     ax, 1
                jne     @@bad_exit          ; if not, we're out of here..

                mov     ax, 1
                push    ax
                call    __parclen           ; how big is the string?
                add     sp,2
                cmp     ax,0
                je      @@bad_exit          ; don't bother with null strings

                mov     cx,ax               ; save the length
                push    cx                  ; because clipper screws it up

                mov     ax, 1
                push    ax
                call    __parc              ; where is the string?
                add     sp,2                ; address in DX:AX
                pop     cx                  ; get the length back

                mov     bx, dx              ; point DS:DX to filespec
                mov     ds, bx              ; can't do a ss -> ss load
                mov     dx, ax              ; on an 80x86


                mov     ah, 4Eh             ; DOS FindFirst
                mov     al, 0               ; as per RB's IL
                mov     cx, HIDDEN + SYSTEM + DIRECTORY  
                int     21h
                jc      @@NOT_FOUND         ; carry set on error


                ; else is found, validate that it's a directory

                mov     ah, 2Fh   ; get DTA in es:bx
                int     21h

                mov     al, DIRECTORY
                and     al, byte ptr es:[bx+15h]  ; is it really a directory?
                push    ax ; save the result

                ;**********************
@@FINDNEXT:     mov    ah, 4Fh              ; run DOS FindNext until 
                int    21h                  ; it returns empty, so we don't                   
                jnc    @@FINDNEXT           ; use up all the NCP searches on 
                ; should only execute once unless user passed a wildcard
                ;**********************

                pop     ax
                cmp     al, 0 
                je      @@NOT_FOUND

@@FOUND:        mov    ax, 1                ; found!
                jmp    short @@BYE

@@BAD_EXIT:
@@NOT_FOUND:    mov    ax, 0                ; not found


@@BYE:          mov    bx, seg dgroup       ; point ds to Clipper's dataseg
                mov    ds, bx               ; so it doesn't get all confused

                push   ax
                call   __retl               ; send the data back to Clipper
                add    sp,2                 ; fix the stack
               

                pop    di
                pop    si
                pop    es
                pop    ds
                pop    bp
                Ret

ISDIR Endp   ; end of procedure

_prog Ends   ; End of code Segment

End          ; end of program
